/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.shape;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.loaders.DataObjectExistsException;
import org.openide.loaders.FileEntry;
import org.openide.loaders.MultiDataObject;
import org.openide.loaders.MultiDataObject.Entry;
import org.openide.loaders.MultiFileLoader;

/**
 *
 * @author Timon Veenstra
 */
public class ShapeFileLoader extends MultiFileLoader {
    /*
     * Manditory files : 
     * .shp — shape format; the feature geometry itself 
     * .shx —shape index format; a positional index of the feature geometry to allow
     * seeking forwards and backwards quickly 
     * .dbf — attribute format; columnar attributes for each shape, in dBase IV format 
     * 
     * Optional files : 
     * .prj — projection format; the coordinate system and projection information, a
     * plain text file describing the projection using well-known text format
     * .sbn and .sbx — a spatial index of the features 
     * .fbn and .fbx — a spatial index of the features for shapefiles that are read-only 
     * .ain and .aih — an attribute index of the active fields in a table or a theme's attribute
     * table 
     * .ixs — a geocoding index for read-write shapefiles 
     * .mxs — a geocoding index for read-write shapefiles (ODB format) 
     * .atx — an attribute index for the .dbf file in the form of shapefile.columnname.atx
     * (ArcGIS 8 and later) 
     * .shp.xml — geospatial metadata in XML format, such as ISO 19115 or other schemas 
     * .cpg — used to specify the code page (only for .dbf) for identifying the character 
     * encoding to be used
     */

    public static final String PRIMARY_EXT = "shp";
    public static final String[] MANDITORY_EXT = {"shx", "dbf"};
    //TODO shp.xml is composite extention and won't be recognized
    public static final String[] OPTIONAL_EXT = {"prj", "sbn", "sbx", "fbn", "fbx", "ain", "aih", "ixs", "mxs", "atx", "shp.xml", "cpg"};
    
    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.field");

    static {
        Arrays.sort(MANDITORY_EXT);
        Arrays.sort(OPTIONAL_EXT);
    }

    public ShapeFileLoader() {
        super(ShapeDataObject.class.getName());
    }

    
    @Override
    protected String actionsContext() {
        return "Loaders/" + ShapeDataObject.MIMETYPE + "/Actions/"; // NOI18N
    }

    @Override
    protected FileObject findPrimaryFile(FileObject fo) {
        // never recognize folders.
        if (fo.isFolder()) {
            return null;
        }
        String ext = fo.getExt();

        // check if (secondary) file has one of the secondary extentions
        if (Arrays.binarySearch(MANDITORY_EXT, ext) >= 0 || Arrays.binarySearch(OPTIONAL_EXT, ext) >= 0 || Arrays.binarySearch(OPTIONAL_EXT, getDoubleExtension(fo)) >= 0 ) {
            LOGGER.log(Level.FINEST, "FileObject {0} has secondary extention, creating secondary entry and returing find brother with primary", fo);
            
            return FileUtil.findBrother(fo, PRIMARY_EXT);
        }

        FileObject shapeFile = findShapePrimaryFile(fo);
        // perform check if manditory brothers exist
        if (shapeFile != null) {
            for (String manditory : MANDITORY_EXT) {
                if (FileUtil.findBrother(shapeFile, manditory) == null) {
                    LOGGER.log(Level.FINEST, "Failed to find brother manditory extention {0} for FileObject {1}", new Object[]{manditory, fo});
                    return null;
                }
            }
            // all manditory extentions found
            return shapeFile;
        }
        LOGGER.log(Level.FINEST, "No primary file found for FileObject {0}", fo);
        return null;
    }

    @Override
    protected MultiDataObject createMultiObject(FileObject primaryFile) throws DataObjectExistsException, IOException {
        List<FileObject> secondary = new ArrayList<>();
        for(String manditory : MANDITORY_EXT){
            FileObject found = FileUtil.findBrother(primaryFile, manditory);
            if(found != null){
                secondary.add(found);
            }
        }
        for(String optional : OPTIONAL_EXT){
            FileObject found = FileUtil.findBrother(primaryFile, optional);
            if(found != null){
                secondary.add(found);
            }
        }
        ShapeDataObject dataObject = new ShapeDataObject(primaryFile, this);
        dataObject.setSecondaryFiles(secondary);
        return dataObject;
    }

    @Override
    protected Entry createPrimaryEntry(MultiDataObject obj, FileObject primaryFile) {
        Entry entry = new FileEntry(obj, primaryFile);
        LOGGER.log(Level.FINEST, "Creating primary entry for {0}", primaryFile.getName());
        return entry;
    }

    @Override
    protected Entry createSecondaryEntry(MultiDataObject obj, FileObject secondaryFile) {
        assert Arrays.binarySearch(MANDITORY_EXT, secondaryFile.getExt()) >= 0 || Arrays.binarySearch(OPTIONAL_EXT, secondaryFile.getExt()) >= 0 || Arrays.binarySearch(OPTIONAL_EXT, getDoubleExtension(secondaryFile)) >= 0 ;
        FileEntry entry = new FileEntry(obj, secondaryFile);
        LOGGER.log(Level.FINEST, "Creating secondary entry for {0}", secondaryFile.getName());
        return entry;
    }

    private FileObject findShapePrimaryFile(FileObject fo) {
        if (fo.getExt().equals(PRIMARY_EXT)) {
            return fo;
        }
        return null;
    }
    
    /**
     * used to get double extensions from a fileObject.
     * Sometimes there are more dot's in a filename this method is able to return the ext of these type of files 
     * @param fileObject the fileOject to check the extension from
     * @return The extension if found or a an empty string if not 
     */
    protected String getDoubleExtension(FileObject fileObject){
        //test if there are more dot's "." in a fileName i.e .shp.xml
        if(fileObject.getNameExt().split("\\.").length > 2){
            return fileObject.getNameExt().substring(fileObject.getNameExt().indexOf(".") + 1);
        }
        return "";
    }
}
