/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.shape;

import java.awt.Color;
import java.awt.Component;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import javax.swing.JComboBox;
import javax.swing.event.ChangeListener;
import nl.cloudfarming.client.field.model.Shape;
import nl.cloudfarming.client.shape.preview.*;
import org.netbeans.api.visual.action.ActionFactory;
import org.netbeans.api.visual.model.ObjectScene;
import org.netbeans.api.visual.model.ObjectSceneEvent;
import org.netbeans.api.visual.model.ObjectSceneEventType;
import org.netbeans.api.visual.model.ObjectSceneListener;
import org.netbeans.api.visual.model.ObjectState;
import org.openide.WizardDescriptor;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;

/**
 * Default PreviewProvider
 *
 * @author Gerben Feenstra
 */
public class ShapePreviewController implements WizardDescriptor.Panel {

    private static final String PROP_SELECTED_NAME_PROPERTY = "selectedNameProperty";
    private ShapePreviewPanel shapePreviewPanel;
    private ShapeDataObject dataObject;
    private final ObjectScene scene = new ObjectScene();
    private final SceneListener sceneListener = new SceneListener();
    private MapLegendPanel mapLegendPanel;
    private ShapePropertiesPanel shapePropertiesPanel = new ShapePropertiesPanel();
    private ShapePropertyMappingPanel shapePropertyMappingPanel = new ShapePropertyMappingPanel();
    private PropertyMappingListener propertyMappingListener = new PropertyMappingListener();
    private PreviewPanelPalette previewPanelPalette;
            
    @NbBundle.Messages({"shape preview attributes no attribute=None","shape preview panel title=Shape import preview"})
    public ShapePreviewController(ShapeDataObject dataObject) {
        this.dataObject = dataObject;
        this.previewPanelPalette = new PreviewPanelPalette(Color.red);
        
        mapLegendPanel = new MapLegendPanel(previewPanelPalette);
                
        scene.getActions().addAction(ActionFactory.createZoomAction());
        scene.getActions().addAction(ActionFactory.createPanAction());

        ShapePreviewPanel.Builder builder = new ShapePreviewPanel.Builder();

        // Shape Preview panel
        ShapesPanel leftPanel = new ShapesPanel(dataObject, scene, previewPanelPalette);
        builder.setLeftPanel(leftPanel);

        // Legend panel

        builder.setMapLegendPanel(mapLegendPanel);

        // Shape selectorPanel

        List<String> attributes = dataObject.getStringAttributes();
        if(attributes != null && !attributes.isEmpty()){
//            attributes = new ArrayList<>();
            attributes.add("");
        }
        
        shapePropertyMappingPanel.setAvailableProperties(attributes);
        shapePropertyMappingPanel.addItemListener(propertyMappingListener);
        builder.setShapePropertyMappingPanel(shapePropertyMappingPanel);

        // Shape properties panel

        builder.setShapePropertiesPanel(shapePropertiesPanel);
        shapePropertiesPanel.addImportTypeListener(propertyMappingListener);
        builder.setMarkAsPanel(new MarkAsPanel());
        shapePreviewPanel = builder.build();
        shapePreviewPanel.setName(Bundle.shape_preview_panel_title());
        shapePreviewPanel.setTitle(Bundle.shape_preview_panel_title());

        scene.addObjectSceneListener(sceneListener, ObjectSceneEventType.OBJECT_SELECTION_CHANGED);

    }

    @Override
    public Component getComponent() {
        return shapePreviewPanel;
    }

    @Override
    public HelpCtx getHelp() {
        return new HelpCtx("nl.cloudfarming.client.field.wizard.import_geometries");
    }

    @Override
    public void readSettings(Object settings) {
    }

    @Override
    public void storeSettings(Object settings) {
        WizardDescriptor wizardDescriptor = (WizardDescriptor) settings;
        wizardDescriptor.putProperty(PROP_SELECTED_NAME_PROPERTY, shapePreviewPanel.getSelectedNameProperty());
    }

    @Override
    public boolean isValid() {
        return true;
    }

    @Override
    public void addChangeListener(ChangeListener l) {
        //throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void removeChangeListener(ChangeListener l) {
        //throw new UnsupportedOperationException("Not supported yet.");
    }
    
    public List<Shape> getImportShapes() {
        List<Shape> shapes = new ArrayList<>();
        for (Shape shape: dataObject.getShapes()){
            if (Shape.ImportType.IMPORT.equals(shape.getImportType())){
                shapes.add(shape);
            }
        }
        return shapes;
    }    
    

    private class SceneListener implements ObjectSceneListener {

        @Override
        public void objectAdded(ObjectSceneEvent event, Object addedObject) {
        }

        @Override
        public void objectRemoved(ObjectSceneEvent event, Object removedObject) {
        }

        @Override
        public void objectStateChanged(ObjectSceneEvent event, Object changedObject, ObjectState previousState, ObjectState newState) {
        }

        @Override
        public void selectionChanged(ObjectSceneEvent event, Set<Object> previousSelection, Set<Object> newSelection) {
            Shape[] shapes = new Shape[newSelection.size()];
            newSelection.toArray(shapes);
            shapePropertiesPanel.setActiveShape(shapes);
        }

        @Override
        public void highlightingChanged(ObjectSceneEvent event, Set<Object> previousHighlighting, Set<Object> newHighlighting) {
        }

        @Override
        public void hoverChanged(ObjectSceneEvent event, Object previousHoveredObject, Object newHoveredObject) {
        }

        @Override
        public void focusChanged(ObjectSceneEvent event, Object previousFocusedObject, Object newFocusedObject) {
        }
    }

    private class PropertyMappingListener implements ItemListener {

        @Override
        public void itemStateChanged(ItemEvent evt) {
            JComboBox cb = (JComboBox) evt.getSource();
            String boxName = cb.getName();

            // Get the affected item
            Object item = evt.getItem();

            if (evt.getStateChange() == ItemEvent.SELECTED) {
                // Item was just selected
                if (ShapePropertyMappingPanel.PROP_COMBOBOX_NAME.equals(boxName)) {
                    for (Shape shape : dataObject.getShapes()) {
                        shape.setNameFromAttribute((String) item);
                    }
                }
                if (ShapePropertiesPanel.PROP_COMBOBOX_IMPORT_TYPE.equals(boxName)) {
                    for (Object o : scene.getSelectedObjects()) {
                        ((Shape) o).setImportType((Shape.ImportType) item);
                    }
                }
            } else if (evt.getStateChange() == ItemEvent.DESELECTED) {
                // Item is no longer selected
            }
        }
    }
}
